/*!
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
import { isFluidBrowserPackage, } from "@fluidframework/container-definitions";
import { ScriptManager } from "./scriptManager";
export class WebCodeLoader {
    constructor(codeResolver, allowList) {
        this.codeResolver = codeResolver;
        this.allowList = allowList;
        this.loadedModules = new Map();
        this.scriptManager = new ScriptManager();
    }
    async seedModule(source, maybeFluidModule) {
        const resolved = await this.codeResolver.resolveCodeDetails(source);
        if (resolved.resolvedPackageCacheId !== undefined
            && this.loadedModules.has(resolved.resolvedPackageCacheId)) {
            return;
        }
        const fluidModule = (maybeFluidModule !== null && maybeFluidModule !== void 0 ? maybeFluidModule : this.load(source));
        if (resolved.resolvedPackageCacheId !== undefined) {
            this.loadedModules.set(resolved.resolvedPackageCacheId, fluidModule);
        }
    }
    async preCache(source) {
        const resolved = await this.codeResolver.resolveCodeDetails(source);
        if (isFluidBrowserPackage(resolved.resolvedPackage)) {
            return this.scriptManager.preCacheFiles(resolved.resolvedPackage.fluid.browser);
        }
    }
    /**
     * @param source - Details of where to find chaincode
     */
    async load(source) {
        const resolved = await this.codeResolver.resolveCodeDetails(source);
        if (resolved.resolvedPackageCacheId !== undefined) {
            const maybePkg = this.loadedModules.get(resolved.resolvedPackageCacheId);
            if (maybePkg !== undefined) {
                return maybePkg;
            }
        }
        const fluidModuleP = this.loadModuleFromResolvedCodeDetails(resolved);
        if (resolved.resolvedPackageCacheId !== undefined) {
            this.loadedModules.set(resolved.resolvedPackageCacheId, fluidModuleP);
        }
        return fluidModuleP;
    }
    async loadModuleFromResolvedCodeDetails(resolved) {
        var _a;
        if (this.allowList !== undefined && !(await this.allowList.testSource(resolved))) {
            throw new Error("Attempted to load invalid code package url");
        }
        if (!isFluidBrowserPackage(resolved.resolvedPackage)) {
            throw new Error(`Package ${resolved.resolvedPackage.name} not a Fluid module.`);
        }
        const loadedScriptsP = this.scriptManager.loadLibrary(resolved.resolvedPackage.fluid.browser.umd);
        let fluidModule;
        for (const script of await loadedScriptsP) {
            const maybeFluidModule = script.entryPoint;
            if (maybeFluidModule.fluidExport !== undefined) {
                if (fluidModule !== undefined) {
                    throw new Error("Multiple Fluid modules loaded");
                }
                fluidModule = maybeFluidModule;
            }
        }
        if (((_a = fluidModule) === null || _a === void 0 ? void 0 : _a.fluidExport) === undefined) {
            throw new Error("Entry point of loaded code package not a Fluid module");
        }
        return fluidModule;
    }
}
//# sourceMappingURL=webLoader.js.map